@@ -1,6 +1,6 @@
 /* Native-dependent code for GNU/Linux on MIPS processors.
 
-   Copyright (C) 2001-2022 Free Software Foundation, Inc.
+   Copyright (C) 2001-2025 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,9 +17,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "command.h"
-#include "gdbcmd.h"
+#include "cli/cli-cmds.h"
 #include "inferior.h"
 #include "mips-tdep.h"
 #include "target.h"
@@ -90,7 +89,7 @@ static mips_linux_nat_target the_mips_linux_nat_target;
 static int have_ptrace_regsets = 1;
 
 /* Map gdb internal register number to ptrace ``address''.
-   These ``addresses'' are normally defined in <asm/ptrace.h>. 
+   These ``addresses'' are normally defined in <asm/ptrace.h>.
 
    ptrace does not provide a way to read (or set) MIPS_PS_REGNUM,
    and there's no point in reading or setting MIPS_ZERO_REGNUM.
@@ -458,6 +457,10 @@ mips_linux_nat_target::read_description ()
 
   if (have_dsp < 0)
     {
+      /* Assume no DSP if there is no inferior to inspect with ptrace.  */
+      if (inferior_ptid == null_ptid)
+	return _MIPS_SIM == _ABIO32 ? tdesc_mips_linux : tdesc_mips64_linux;
+
       int tid = get_ptrace_pid (inferior_ptid);
 
       errno = 0;
@@ -512,7 +515,7 @@ mips_show_dr (const char *func, CORE_ADDR addr,
   if (addr || len)
     gdb_printf (gdb_stdlog,
 		" (addr=%s, len=%d, type=%s)",
-		paddress (target_gdbarch (), addr), len,
+		paddress (current_inferior ()->arch (), addr), len,
 		type == hw_write ? "data-write"
 		: (type == hw_read ? "data-read"
 		   : (type == hw_access ? "data-read/write"
@@ -522,10 +525,10 @@ mips_show_dr (const char *func, CORE_ADDR addr,
 
   for (i = 0; i < MAX_DEBUG_REGISTER; i++)
     gdb_printf (gdb_stdlog, "\tDR%d: lo=%s, hi=%s\n", i,
-		paddress (target_gdbarch (),
+		paddress (current_inferior ()->arch (),
 			  mips_linux_watch_get_watchlo (&watch_mirror,
 							i)),
-		paddress (target_gdbarch (),
+		paddress (current_inferior ()->arch (),
 			  mips_linux_watch_get_watchhi (&watch_mirror,
 							i)));
 }
@@ -559,7 +562,7 @@ mips_linux_nat_target::can_use_hw_breakpoint (enum bptype type,
     default:
       return 0;
     }
- 
+
   for (i = 0;
        i < mips_linux_watch_get_num_valid (&watch_readback) && cnt;
        i++)
@@ -633,9 +636,9 @@ mips_linux_nat_target::region_ok_for_hw_watchpoint (CORE_ADDR addr, int len)
 static int
 write_watchpoint_regs (void)
 {
-  for (const lwp_info *lp : all_lwps ())
+  for (const lwp_info &lp : all_lwps ())
     {
-      int tid = lp->ptid.lwp ();
+      int tid = lp.ptid.lwp ();
       if (ptrace (PTRACE_SET_WATCH_REGS, tid, &watch_mirror, NULL) == -1)
 	perror_with_name (_("Couldn't write debug register"));
     }
@@ -781,9 +784,7 @@ mips_linux_nat_target::close ()
   linux_nat_trad_target::close ();
 }
 
-void _initialize_mips_linux_nat ();
-void
-_initialize_mips_linux_nat ()
+INIT_GDB_FILE (mips_linux_nat)
 {
   add_setshow_boolean_cmd ("show-debug-regs", class_maintenance,
 			   &show_debug_regs, _("\
