@@ -1458,6 +1458,10 @@ struct symbol : public general_symbol_info, public allocate_on_obstack<symbol>
 
   void set_symtab (struct symtab *symtab);
 
+  /* Relocate this symbol.  OFFSETS is the relocation offsets to use.  */
+
+  void relocate (gdb::array_view<const CORE_ADDR> offsets);
+
   /* Data type of value */
 
   struct type *m_type = nullptr;
@@ -1565,16 +1569,6 @@ struct block_symbol
 
 #define SYMBOL_LOCATION_BATON(symbol)   (symbol)->aux_value
 
-inline const block *
-symbol::value_block () const
-{
-  if (const symbol_block_ops *block_ops = this->block_ops ();
-      block_ops != nullptr && block_ops->get_block_value != nullptr)
-    return block_ops->get_block_value (this);
-
-  return m_value.block;
-}
-
 extern int register_symbol_computed_impl (location_class,
 					  const struct symbol_computed_ops *);
 
@@ -1685,29 +1679,29 @@ struct linetable
   struct linetable_entry item[1];
 };
 
-/* How to relocate the symbols from each section in a symbol file.
-   The ordering and meaning of the offsets is file-type-dependent;
-   typically it is indexed by section numbers or symbol types or
-   something like that.  */
-
-typedef std::vector<CORE_ADDR> section_offsets;
-
 /* Each source file or header is represented by a struct symtab.
    The name "symtab" is historical, another name for it is "filetab".
    These objects are chained through the `next' field.  */
 
 struct symtab
 {
+  symtab (struct compunit_symtab *cust, const char *filename,
+	  const char *filename_for_id, enum language language)
+    : m_filename (filename),
+      m_filename_for_id (filename_for_id),
+      m_compunit (cust),
+      m_language (language)
+  {
+    gdb_assert (m_filename != nullptr);
+    gdb_assert (m_filename_for_id != nullptr);
+    gdb_assert (m_compunit != nullptr);
+  }
+
   struct compunit_symtab *compunit () const
   {
     return m_compunit;
   }
 
-  void set_compunit (struct compunit_symtab *compunit)
-  {
-    m_compunit = compunit;
-  }
-
   const struct linetable *linetable () const
   {
     return m_linetable;
@@ -1749,16 +1743,24 @@ struct symtab
     m_fullname = name.release ();
   }
 
+  const char *filename () const
+  { return m_filename; }
+
+  const char *filename_for_id () const
+  { return m_filename_for_id; }
+
   /* Unordered chain of all filetabs in the compunit,  with the exception
      that the "main" source file is the first entry in the list.  */
 
-  struct symtab *next;
+  struct symtab *next = nullptr;
+
+private:
 
   /* Name of this source file, in a form appropriate to print to the user.
 
-     This pointer is never nullptr.  */
+     This pointer is never nullptr and is set from the constructor.  */
 
-  const char *filename;
+  const char *m_filename;
 
   /* Filename for this source file, used as an identifier to link with
      related objects such as associated macro_source_file objects.  It must
@@ -1767,28 +1769,29 @@ struct symtab
      follow that rule, or another form of the same file name, this is up to
      the specific debug info reader.
 
-     This pointer is never nullptr.*/
-  const char *filename_for_id;
+     This pointer is never nullptr, and is set from the constructor.  */
+  const char *m_filename_for_id;
 
-private:
+  /* Backlink to containing compunit symtab.
 
-  /* Backlink to containing compunit symtab.  */
+     This pointer is never nullptr, and is set from the constructor.  */
 
   struct compunit_symtab *m_compunit;
 
   /* Table mapping core addresses to line numbers for this file.
      Can be NULL if none.  Never shared between different symtabs.  */
 
-  const struct linetable *m_linetable;
+  const struct linetable *m_linetable = nullptr;
 
-  /* Language of this source file.  */
+  /* Language of this source file.  This is set in the object
+     constructor.  */
 
   enum language m_language;
 
   /* Full name of file as found by searching the source path.
      NULL if not yet known.  */
 
-  char *m_fullname;
+  char *m_fullname = nullptr;
 };
 
 /* A range adapter to allowing iterating over all the file tables in a list.  */
@@ -1957,6 +1960,10 @@ struct compunit_symtab : intrusive_list_node<compunit_symtab>
   /* Clear any cached source file names.  */
   void forget_cached_source_info ();
 
+  /* Return symbol at ADDR or NULL if no symbol is found.  Only exact matches
+     for ADDR are considered.  */
+  struct symbol *symbol_at_address (CORE_ADDR addr) const;
+
   /* Object file from which this symtab information was read.  */
   struct objfile *m_objfile;
 
